<?php
require_once 'BenutzerStatistik_Summarizer.class.php';
require_once 'BenutzerStatistik_Helper.class.php';
require_once 'vendor/flexi/flexi.php';
require_once 'lib/include/reiter.inc.php';

/**
 * BenutzerStatistikPlugin.class.php
 *
 * Diese Klasse stellt das Hauptplugin der Benutzerstatistik dar.
 *
 * @author  Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @copyright IBIT 2007, 2008
 * @version 0.8.8
 */

class BenutzerStatistikPlugin extends AbstractStudIPSystemPlugin
{
	public $user;
	private $permissions;
	private $template_factory;
	private $db;
	private $tabs = array(
		'show'=>'Gesamt',
		'today'=>'Heute',
		'yesterday'=>'Gestern',
		'monthly'=>'Monatlich',
		'quarterly'=>'Quartalsweise',
		'yearly'=>'Jhrlich',
		'admin'=>'Administration',
	);
	private static $hit_once = false;

	public function __construct()
	{
		parent::AbstractStudIPSystemPlugin();

		$this->user = $this->getUser();
		$this->permissions = $this->user->getPermission();
		if ($this->permissions->hasRootPermission())
		{
			$nav = new PluginNavigation();
			$nav->setDisplayname($this->getPluginname());
			$this->setNavigation($nav);

			$this->setPluginiconname('images/stats.gif');
		}
		$this->template_factory = new Flexi_TemplateFactory(dirname(__FILE__).'/templates');
		$this->db = new DB_Seminar;

		$path = str_replace($GLOBALS['ABSOLUTE_PATH_STUDIP'], '', dirname(__FILE__));
		$GLOBALS['_include_additional_header'] .= '<link href="'.$path.'/css/statistics.css" type="text/css" rel="stylesheet"/>'."\n";
		$GLOBALS['_include_additional_header'] .= '<script type="text/javascript" src="'.$path.'/js/javascript.js"></script>'."\n";

		if (!empty($GLOBALS['STUDIP_INSTALLATION_ID']) and $GLOBALS['STUDIP_INSTALLATION_ID']=='uol')
		{
			$this->tabs['munin'] = 'Munin';
		}
	}

	public function getPluginname()
	{
		return _('Benutzer-Statistik');
	}

	public function hasBackgroundTasks()
	{
		return true;
	}

	public function doBackgroundTasks()
	{
		if (!empty($_POST['sniffer_ping']))
		{
			$this->StoreHit();
		}
		elseif (self::$hit_once)
		{
			return;
		}

		$user_agent = empty($_SERVER['HTTP_USER_AGENT']) ? '?' : $_SERVER['HTTP_USER_AGENT'];

		$permission = 'unknown';
		if ($this->permissions->hasAdminPermission() or $this->permissions->hasRootPermission())
		{
			$permission = 'admin';
		}
		elseif ($this->permissions->hasTeacherPermission())
		{
			$permission = 'teacher';
		}
		elseif ($this->permissions->hasTutorPermission() or $this->permissions->hasStudentPermission())
		{
			$permission = 'student';
		}

		$this->db->queryf("INSERT INTO user_statistics (ip, user_agent, daystamp, hash, permission) VALUES (MD5(INET_ATON('%s')), '%s', NOW(), MD5(CONCAT('%s', '%s')), '%s') ON DUPLICATE KEY UPDATE hits=hits+1", $_SERVER['REMOTE_ADDR'], $user_agent, $this->user->getUserid(), $this->user->getUsername(), $permission);
		// Dummy entry for statistic purposes
		$this->db->queryf("INSERT IGNORE INTO user_statistics_uniqueusers (hash, permission, visits, hits) VALUES (MD5(CONCAT('%s', '%s')), '%s', 0, 0)", $this->user->getUserid(), $this->user->getUsername(), $permission);

		$this->db->queryf("SELECT COUNT(*) AS quantity FROM user_statistics WHERE ip=MD5(INET_ATON('%s')) AND user_agent='%s' AND daystamp=NOW() AND hash=MD5(CONCAT('%s', '%s')) AND (hits>5 OR javascript=1)", $_SERVER['REMOTE_ADDR'], $user_agent, $this->user->getUserid(), $this->user->getUsername());
		if (!$this->db->next_record() or $this->db->f('quantity')==0)
		{
			$uri = 'http'.($_SERVER['SERVER_PORT']!=80?'s':'').'://'.$_SERVER['HTTP_HOST'].dirname($_SERVER['REQUEST_URI']).'/index.php';

			$template = $this->template_factory->open('sniffer');
			$template->set_attribute('link', $uri);
			echo $template->render();
		}
		$this->db->free();

		self::$hit_once = true;
	}

	private function StoreHit()
	{
		ob_end_clean();

		$user_agent = empty($_SERVER['HTTP_USER_AGENT']) ? '?' : $_SERVER['HTTP_USER_AGENT'];
		$this->db->queryf("UPDATE user_statistics SET javascript=1, screen_width=%d, screen_height=%d WHERE ip=MD5(INET_ATON('%s')) AND user_agent='%s' AND hash=MD5(CONCAT('%s', '%s')) AND daystamp=NOW()", $_POST['width'], $_POST['height'], $_SERVER['REMOTE_ADDR'], $user_agent, $this->user->getUserid(), $this->user->getUsername());

		die;
	}

	private function displayHeader()
	{
		if (!$this->permissions->hasRootPermission())
		{
			echo _('Unberechtigter Zugriff');
			page_close();
			die;
		}

		$structure = array();
		foreach ($this->tabs as $cmd=>$title)
		{
			$structure[$cmd] = array(
				'topKat'=>'',
				'name'=>_($title),
				'link'=>PluginEngine::getLink($this, array(), $cmd),
				'active'=>($cmd==$_REQUEST['cmd'])
			);
		}

		$reiter = new reiter;
		$reiter->create($structure, $_REQUEST['cmd']);
		echo '<br/>';
	}

	public function actionmunin()
	{
		if (empty($GLOBALS['STUDIP_INSTALLATION_ID']) or $GLOBALS['STUDIP_INSTALLATION_ID']!='uol')
		{
			return;
		}
		$this->displayHeader();
		echo gzuncompress(base64_decode('eNpljs0OwiAQhF+FcC+g8WCw9F1aWWEToGRZ/Hl7iR57nG/yZWbGB60ZRKO7k5G5Wq1jD1jUE4lVLzjtyUPZOgXlQedBij5gSLDSaMLBmBp3j1VFzkmK39i2kwdy0kjR+JPAyRd6jlaczPlS3zcRAUNkK67GjCiXWf9fLl++jT2d'));
	}

	public function actionadmin()
	{
		$this->displayHeader();

		$this->db->queryf("SELECT COUNT(DISTINCT daystamp) AS quantity FROM user_statistics WHERE daystamp<=LAST_DAY(TIMESTAMPADD(MONTH, -1, NOW())) AND daystamp<=TIMESTAMPADD(DAY, -2, NOW())");
		$days_to_summarize = $this->db->next_record() ? $this->db->f('quantity') : 0;
		$this->db->free();

		$template = $this->template_factory->open('admin');
		$template->set_attribute('days_to_summarize', $days_to_summarize);
		$template->set_attribute('links', array(
			'summarize'=>PluginEngine::getLink($this, array(), 'summarize')
		));
		echo $template->render();
		// some more VOODOO2FOLLOW
	}

	public function actionsummarize()
	{
		new BenutzerStatistik_Summarizer();

		Header('Location: '.PluginEngine::getLink($this, array(), 'admin'));
		die;
	}

	public function actionshow()
	{
		$this->displayStatistics();
	}

	public function actiontoday()
	{
		$this->displayStatistics(time());
	}

	public function actionyesterday()
	{
		$this->displayStatistics(strtotime('yesterday'));
	}

	private function displayStatistics($timestamp=null)
	{
		$this->displayHeader();

		$template = $this->template_factory->open('stats');
		$template->set_attribute('stats', $this->getStatistics($timestamp));
		$template->set_attribute('link', PluginEngine::getLink($this, array()).'&agent=');
		$template->set_attribute('icon_path', str_replace($GLOBALS['ABSOLUTE_PATH_STUDIP'], '', dirname(__FILE__)).'/images/');
		echo $template->render();
	}

	public function actionmonthly()
	{
		$month_names = $this->getMonthNames();

		$this->displayHeader();

		if (empty($_GET['month']))
		{
			$month = date('n');
			$year = date('Y');
		}
		else
		{
			list($year, $month) = explode('-', $_GET['month']);
		}

		$months = array();
		$this->db->query("SELECT DISTINCT month_stamp FROM (SELECT DISTINCT DATE_FORMAT(daystamp, '%Y-%c') AS month_stamp FROM user_statistics UNION SELECT DISTINCT DATE_FORMAT(daystamp, '%Y-%c') AS month_stamp FROM user_statistics_daily) AS tmp_table ORDER BY month_stamp DESC");
		while ($this->db->next_record())
		{
			list($y, $m) = explode('-', $this->db->f('month_stamp'));
			$months[$this->db->f('month_stamp')] = $month_names[$m].' '.$y;
		}
		$this->db->free();

		$stats = array();
		$totals = array(
			'hits'=>0,
			'visits'=>0
		);
		$this->db->queryf("SELECT DAY(daystamp) AS day, permission, SUM(hits) AS hits, COUNT(hash) AS visits, COUNT(DISTINCT hash) AS headcount FROM user_statistics WHERE daystamp BETWEEN FROM_UNIXTIME(%d) AND FROM_UNIXTIME(%d) GROUP BY daystamp, permission UNION SELECT DAY(daystamp) AS day, permission, SUM(hits) AS hits, SUM(visits) AS visits, SUM(unique_visits) AS headcount FROM user_statistics_daily WHERE daystamp BETWEEN FROM_UNIXTIME(%d) AND FROM_UNIXTIME(%d) GROUP BY daystamp, permission", mktime(0,0,0,$month,1,$year), mktime(0,0,0,$month+1,1,$year)-1, mktime(0,0,0,$month,1,$year), mktime(0,0,0,$month+1,1,$year)-1);
		while ($this->db->next_record())
		{
			if (empty($stats[$this->db->f('day')]))
			{
				$stats[$this->db->f('day')] = array('total'=>array(
					'hits'=>0,
					'visits'=>0,
					'headcount'=>0
				));
			}
			$stats[$this->db->f('day')][$this->db->f('permission')] = array(
				'hits'=>$this->db->f('hits'),
				'visits'=>$this->db->f('visits'),
				'headcount'=>$this->db->f('headcount')
			);
			$stats[$this->db->f('day')]['total']['hits'] += $this->db->f('hits');
			$stats[$this->db->f('day')]['total']['visits'] += $this->db->f('visits');
			$stats[$this->db->f('day')]['total']['headcount'] += $this->db->f('headcount');
			$totals['hits'] += $this->db->f('hits');
			$totals['visits'] += $this->db->f('visits');
		}
		$this->db->free();

		$max = array();
		$average = array(
			'visits'=>0,
			'headcount'=>0,
			'hits'=>0
		);
		$average_count = 0;
		foreach ($stats as $stat)
		{
			foreach ($stat as $permission)
			{
				foreach ($permission as $index=>$value)
				{
					if (empty($max[$index]))
					{
						$max[$index] = 0;
					}
					$max[$index] = max($max[$index], $value);

					$average[$index] += $value;
				}
				$average_count++;
			}
		}
		foreach ($average as $index=>$value)
		{
			$average[$index] = $average_count>0 ? $value/$average_count : 0;
		}

		// TODO: Hier anpassen
		$this->db->queryf("SELECT MAX(headcount) AS headcount FROM (SELECT COUNT(DISTINCT hash) AS headcount FROM user_statistics WHERE daystamp BETWEEN FROM_UNIXTIME(%d) AND FROM_UNIXTIME(%d) UNION SELECT unique_visits AS headcount FROM user_statistics_monthly WHERE month_stamp=LAST_DAY(FROM_UNIXTIME(%d))) AS tmp_table", mktime(0,0,0,$month,1,$year), mktime(0,0,0,$month+1,1,$year)-1, mktime(0,0,0,$month,1,$year));
		$totals['headcount'] = $this->db->next_record() ? $this->db->f('headcount') : '?';
		$this->db->free();

		$template = $this->template_factory->open('monthly');
		$template->set_attribute('type', empty($_REQUEST['type']) ? 'visits' : $_REQUEST['type']);
		$template->set_attribute('months', $months);
		$template->set_attribute('month_stamp', mktime(0,0,0, $month, 1, $year));
		$template->set_attribute('stats', $stats);
		$template->set_attribute('max', $max);
		$template->set_attribute('totals', $totals);
		$template->set_attribute('average', $average);
		$template->set_attribute('visits_scale', $this->getScale($max['visits']));
		$template->set_attribute('hits_scale', $this->getScale($max['hits']));
		$template->set_attribute('image_path', str_replace($GLOBALS['ABSOLUTE_PATH_STUDIP'], '', dirname(__FILE__)).'/images/');
		echo $template->render();
	}

	public function actionquarterly()
	{
		$this->displayHeader();

		$quarters = array();

		$this->db->queryf("SELECT DISTINCT quarter_stamp FROM (SELECT DISTINCT CONCAT(YEAR(daystamp),'-',CEIL(MONTH(daystamp)/3)) AS quarter_stamp FROM user_statistics UNION SELECT DISTINCT CONCAT(YEAR(daystamp),'-',CEIL(MONTH(daystamp)/3)) AS quarter_stamp FROM user_statistics_daily) AS tmp_table ORDER BY quarter_stamp DESC");
		while ($this->db->next_record())
		{
			list($year, $quarter) = explode('-', $this->db->f('quarter_stamp'));
			$quarters[$this->db->f('quarter_stamp')] = array(
				'quarter'=>$quarter,
				'year'=>$year
			);
		}
		$this->db->free();

		if (empty($_GET['quarter']))
		{
			$quarter = ceil(date('m')/3);
			$year = date('Y');
		}
		else
		{
			list ($year, $quarter) = explode('-', $_GET['quarter']);
		}
		$quarter_stamp = mktime(0,0,0, $quarter*3-2, 1, $year);

		$stats = array();
		$this->db->queryf("SELECT day, hits, visits, headcount FROM (SELECT DATE_FORMAT(daystamp, '%%c-%%e') AS day, SUM(hits) AS hits, COUNT(hash) AS visits, COUNT(DISTINCT hash) AS headcount FROM user_statistics WHERE daystamp BETWEEN FROM_UNIXTIME(%d) AND FROM_UNIXTIME(%d) GROUP BY daystamp UNION SELECT DATE_FORMAT(daystamp, '%%c-%%e') AS day, SUM(hits) AS hits, SUM(visits) AS visits, SUM(unique_visits) AS headcount FROM user_statistics_daily WHERE daystamp BETWEEN FROM_UNIXTIME(%d) AND FROM_UNIXTIME(%d) GROUP BY daystamp) AS tmp_table", $quarter_stamp, strtotime('3 months', $quarter_stamp)-1, $quarter_stamp, strtotime('3 months', $quarter_stamp)-1);
		while ($this->db->next_record())
		{
			$stats[$this->db->f('day')] = array(
				'hits'=>$this->db->f('hits'),
				'visits'=>$this->db->f('visits'),
				'headcount'=>$this->db->f('headcount')
			);
		}
		$this->db->free();

		$max = array();
		$totals = array();
		$average = array(
			'visits'=>0,
			'headcount'=>0,
			'hits'=>0
		);
		$average_count = 0;
		foreach ($stats as $stat)
		{
			foreach ($stat as $index=>$value)
			{
				if (empty($totals[$index]))
				{
					$totals[$index] = 0;
				}
				$totals[$index] += $value;

				if (empty($max[$index]))
				{
					$max[$index] = 0;
				}
				$max[$index] = max($max[$index], $value);

				$average[$index] += $value;
			}
			$average_count++;
		}
		foreach ($average as $index=>$value)
		{
			$average[$index] = $value/$average_count;
		}

		$this->db->queryf("SELECT COUNT(DISTINCT hash) AS headcount FROM (SELECT hash FROM user_statistics WHERE daystamp BETWEEN FROM_UNIXTIME(%d) AND FROM_UNIXTIME(%d) UNION SELECT hash FROM user_statistics_uniqueusers_monthly WHERE month BETWEEN %d AND %d AND year=%d) AS tmp_table", $quarter_stamp, strtotime('3 months', $quarter_stamp)-1, ceil(date('m', $quarter_stamp)/3)*3, ceil(date('m', $quarter_stamp)/3)*3+2, date('Y', $quarter_stamp));
		$totals['headcount'] = $this->db->next_record() ? $this->db->f('headcount') : '?';
		$this->db->free();

		$days = date('t', $quarter_stamp)+date('t', strtotime('1 month', $quarter_stamp))+date('t', strtotime('2 months', $quarter_stamp));

		$day_captions = array();
		$current_caption = array(
			'title'=>'1',
			'span'=>1
		);
		for ($i=1, $stamp=strtotime('tomorrow', $quarter_stamp); $i<$days; $i++, $stamp=strtotime('tomorrow', $stamp))
		{
			$day = date('j', $stamp);
			if ($day%5==0 or $day==1)
			{
				array_push($day_captions, $current_caption);
				$current_caption = array(
					'title'=>date('j', $stamp),
					'span'=>0
				);
			}
			$current_caption['span']+=1;
		}
		if ($current_caption['span']!=0)
		{
			array_push($day_captions, $current_caption);
		}

		$template = $this->template_factory->open('quarterly');
		$template->set_attribute('type', empty($_REQUEST['type']) ? 'visits' : $_REQUEST['type']);
		$template->set_attribute('links', array(
			'submit'=>PluginEngine::getLink($this, array(), 'quarterly'),
			'monthly'=>PluginEngine::getLink($this, array(), 'monthly')
		));
		$template->set_attribute('quarters', $quarters);
		$template->set_attribute('quarter_stamp', $quarter_stamp);
		$template->set_attribute('days', $days);
		$template->set_attribute('td_width', round(95/$days, 2));
		$template->set_attribute('stats', $stats);
		$template->set_attribute('totals', $totals);
		$template->set_attribute('average', $average);
		$template->set_attribute('max', $max);
		$template->set_attribute('day_captions', $day_captions);
		$template->set_attribute('visits_scale', $this->getScale($max['visits']));
		$template->set_attribute('hits_scale', $this->getScale($max['hits']));
		$template->set_attribute('months', $this->getMonthNames());
		$template->set_attribute('image_path', str_replace($GLOBALS['ABSOLUTE_PATH_STUDIP'], '', dirname(__FILE__)).'/images/');
		echo $template->render();
	}

	public function actionyearly()
	{
		$this->displayHeader();

		$years = array();
		$this->db->queryf("SELECT DISTINCT year FROM (SELECT DISTINCT YEAR(daystamp) AS year FROM user_statistics UNION SELECT DISTINCT YEAR(month_stamp) AS year FROM user_statistics_monthly) AS tmp_table ORDER BY year DESC");
		while ($this->db->next_record())
		{
			array_push($years, $this->db->f('year'));
		}
		$this->db->free();

		$year = empty($_GET['year']) ? date('Y') : $_GET['year'];

		$totals = $max = $average = array(
			'visits'=>0,
			'uniquevisits'=>0,
			'hits'=>0
		);

		$months = array();
		$this->db->queryf("SELECT month, permission, visits, uniquevisits, hits FROM (SELECT MONTH(daystamp) AS month, permission, COUNT(*) AS visits, COUNT(DISTINCT hash) AS uniquevisits, SUM(hits) AS hits FROM user_statistics WHERE YEAR(daystamp)=%d GROUP BY MONTH(daystamp), permission UNION SELECT MONTH(month_stamp) AS month, permission, SUM(visits) AS visits, SUM(unique_visits) AS uniquevisits, SUM(hits) AS hits FROM user_statistics_monthly WHERE YEAR(month_stamp)=%d GROUP BY MONTH(month_stamp), permission) AS tmp_table", $year, $year);
		while ($this->db->next_record())
		{
			if (!isset($months[$this->db->f('month')]))
			{
				$months[$this->db->f('month')] = array(
					'total'=>array(
						'visits'=>0,
						'unique_visits'=>0,
						'hits'=>0
					)
				);
			}
			$months[$this->db->f('month')][$this->db->f('permission')] = array(
				'visits'=>$this->db->f('visits'),
				'uniquevisits'=>$this->db->f('uniquevisits'),
				'hits'=>$this->db->f('hits')
			);
			$months[$this->db->f('month')]['total']['visits'] += $this->db->f('visits');
			$months[$this->db->f('month')]['total']['uniquevisits'] += $this->db->f('uniquevisits');
			$months[$this->db->f('month')]['total']['hits'] += $this->db->f('hits');
			$max['visits'] = max($max['visits'], $this->db->f('visits'));
			$max['uniquevisits'] = max($max['uniquevisits'], $this->db->f('uniquevisits'));
			$max['hits'] = max($max['hits'], $this->db->f('hits'));
			$average['visits'] += $this->db->f('visits');
			$average['uniquevisits'] += $this->db->f('uniquevisits');
			$average['hits'] += $this->db->f('hits');
			$totals['visits'] += $this->db->f('visits');
			$totals['hits'] += $this->db->f('hits');
		}
		$this->db->free();

		foreach ($average as $index=>$value)
		{
			$average[$index] = $value/count($months);
		}

		$this->db->queryf("SELECT COUNT(DISTINCT hash) AS uniquevisits FROM (SELECT DISTINCT hash FROM user_statistics WHERE YEAR(daystamp)=%d UNION SELECT DISTINCT hash FROM user_statistics_uniqueusers_monthly WHERE year=%d) AS tmp_table", $year, $year);
		$totals['uniquevisits'] = $this->db->next_record() ? $this->db->f('uniquevisits') : 0;
		$this->db->free();

		$template = $this->template_factory->open('yearly');
		$template->set_attribute('months', $months);
		$template->set_attribute('years', $years);
		$template->set_attribute('year', $year);
		$template->set_attribute('max', $max);
		$template->set_attribute('average', $average);
		$template->set_attribute('totals', $totals);
		$template->set_attribute('links', array(
			'monthly'=>PluginEngine::getLink($this, array(), 'monthly')
		));
		$template->set_attribute('monthnames', $this->getMonthNames());
		$template->set_attribute('image_path', str_replace($GLOBALS['ABSOLUTE_PATH_STUDIP'], '', dirname(__FILE__)).'/images/');
		echo $template->render();
	}

	private function getScale($max)
	{
		$possible_scales = array(10, 25, 50);
		$test_scales = $possible_scales;

		do
		{
			$test_scale = array_shift($test_scales);
			if (empty($test_scales))
			{
				foreach ($possible_scales as $index=>$scale)
				{
					$possible_scales[$index] = $scale*10;
				}
				$test_scales = $possible_scales;
			}
		}
		while ($max/$test_scale>10);

		$scale = array();
		$current_scale = 0;
		while ($current_scale<$max)
		{
			array_push($scale, array(
				'value'=>$current_scale,
				'percent'=>$current_scale/$max
			));
			$current_scale += $test_scale;
		}

		return $scale;
	}

	private function getStatistics($date=null, $end_date=null)
	{
		$query_append = '';
		if ($date!==null and $end_date===null)
		{
			$query_append .= " daystamp=DATE(FROM_UNIXTIME(".$date."))";
		}
		elseif ($date!==null and $end_date!==null)
		{
			$query_append .= " daystamp BETWEEN DATE(FROM_UNIXTIME(".$date.")) AND DATE(FROM_UNIXTIME(".$end_date."))";
		}

		$query = "SELECT COUNT(DISTINCT hash) AS headcount, DATE_FORMAT(MIN(daystamp), '%%d.%%m.%%Y') AS start_date, DATE_FORMAT(MAX(daystamp), '%%d.%%m.%%Y') AS end_date, COUNT(*) AS visits, SUM(hits) AS hits, SUM(javascript) AS javascript FROM user_statistics";
		if (!empty($query_append))
		{
			$query .= " WHERE".$query_append;
		}
		$this->db->queryf($query);
		$this->db->next_record();
		$stats = array(
			'date'=> ($date!==null and $end_date===null)
				? date('d.m.Y', $date)
				: array(
					'start'=>$this->db->f('start_date'),
					'end'=>$this->db->f('end_date')
					),
			'visits'=>$this->db->f('visits'),
			'headcount'=>$this->db->f('headcount'),
			'hits'=>$this->db->f('hits'),
			'javascript'=>$this->db->f('javascript'),
			'screen_sizes'=>array(),
			'user_agents'=>array(),
			'permissions'=>array()
		);
		$this->db->free();

		$this->db->queryf("SELECT DATE_FORMAT(MIN(daystamp), '%%d.%%m.%%Y') AS start_date FROM user_statistics_daily".(empty($query_append)?'':' WHERE'.$query_append));
		if ($this->db->next_record() and ($date===null or $end_date!==null))
		{
			$stats['date']['start'] = $this->db->f('start_date');
		}
		$this->db->free();

		$query = "SELECT COUNT(*) AS quantity, COUNT(DISTINCT hash) AS headcount, permission FROM user_statistics";
		if (!empty($query_append))
		{
			$query .= " WHERE".$query_append;
		}
		$query .= " GROUP BY permission";
		$this->db->queryf($query);
		while ($this->db->next_record())
		{
			$stats['permissions'][$this->db->f('permission')] = $this->db->f('quantity');
			$stats['permissions'][$this->db->f('permission').'_headcount'] = $this->db->f('headcount');
		}
		$this->db->free();

		$query = "SELECT COUNT(*) AS quantity, screen_width, screen_height FROM user_statistics WHERE screen_width IS NOT NULL AND screen_height IS NOT NULL ".(empty($query_append)?'':" AND".$query_append)."GROUP BY screen_width, screen_height";
		$this->db->queryf($query);
		while ($this->db->next_record())
		{
			$stats['screen_sizes'][$this->db->f('screen_width').'x'.$this->db->f('screen_height')] = array(
				'width'=>$this->db->f('screen_width'),
				'height'=>$this->db->f('screen_height'),
				'quantity'=>$this->db->f('quantity')
			);
		}
		$this->db->free();

		if ($date===null and $end_date===null)
		{
			$this->db->queryf("SELECT width, height, quantity FROM user_statistics_screensizes WHERE width>0 AND height>0");
			while ($this->db->next_record())
			{
				if (!isset($stats['screen_sizes'][$this->db->f('width').'x'.$this->db->f('height')]))
				{
					$stats['screen_sizes'][$this->db->f('width').'x'.$this->db->f('height')] = array(
						'width'=>$this->db->f('width'),
						'height'=>$this->db->f('height'),
						'quantity'=>0
					);
				}
				$stats['screen_sizes'][$this->db->f('width').'x'.$this->db->f('height')]['quantity'] += $this->db->f('quantity');
			}
			$this->db->free();
		}
		usort($stats['screen_sizes'], create_function('$a,$b', 'return $b["quantity"]-$a["quantity"];'));
		$stats['screen_sizes'] = array_slice($stats['screen_sizes'], 0, 10);

		$query = "SELECT user_agent, COUNT(*) AS quantity FROM user_statistics ".(empty($query_append)?'':" WHERE".$query_append)."GROUP BY user_agent";
		$this->db->queryf($query);
		$user_agents = array();
		while ($this->db->next_record())
		{
			array_push($user_agents, array(
				'user_agent'=>$this->db->f('user_agent'),
				'quantity'=>$this->db->f('quantity')
			));
		}
		$this->db->free();

		if ($date===null and $end_date===null)
		{
			$stats['headcount'] = 0;

			$this->db->queryf("SELECT SUM(visits) AS visits, COUNT(*) unique_visits, SUM(hits) AS hits, permission FROM user_statistics_uniqueusers GROUP BY permission");
			while ($this->db->next_record())
			{
				$stats['headcount'] += $this->db->f('unique_visits');
				$stats['visits'] += $this->db->f('visits');
				$stats['hits'] += $this->db->f('hits');

				$stats['permissions'][$this->db->f('permission')] += $this->db->f('visits');
				$stats['permissions'][$this->db->f('permission').'_headcount'] = $this->db->f('unique_visits');
			}
			$this->db->free();

			$this->db->queryf("SELECT SUM(javascript) AS javascript FROM user_statistics_monthly");
			$stats['javascript'] += $this->db->next_record() ? $this->db->f('javascript') : 0;
			$this->db->free();

			$result = array();
			$this->db->queryf("SELECT os, quantity FROM user_statistics_os");
			while ($this->db->next_record())
			{
				$result[$this->db->f('os')] = array(
					'name'=>$this->db->f('os'),
					'quantity'=>$this->db->f('quantity')
				);
			}
			$this->db->free();
			$stats['os'] = BenutzerStatistik_Helper::parseOperatingSystems($user_agents, $result);

			$result = array();
			$this->db->queryf("SELECT agent_hash, browser, version, user_agent, quantity FROM user_statistics_browser");
			while ($this->db->next_record())
			{
				$quantity = $this->db->f('quantity');
				if ($this->db->f('browser')===null and $this->db->f('version')===null)
				{
					$agent = _('unbekannt');
					$version = $this->db->f('user_agent');
				}
				else
				{
					$agent = $this->db->f('browser');
					$version = $this->db->f('version');
				}
				$agent_hash = md5($agent);

				if (!isset($result[$agent_hash]))
				{
					$result[$agent_hash] = array(
						'name'=>$agent,
						'versions'=>array(),
						'quantity'=>0
					);
				}
				if (!isset($result[$agent_hash]['versions'][$version]))
				{
					$result[$agent_hash]['versions'][$version] = 0;
				}
				$result[$agent_hash]['quantity'] += $quantity;
				$result[$agent_hash]['versions'][$version] += $quantity;
			}
			$this->db->free();

			$stats['user_agents'] = BenutzerStatistik_Helper::parseUserAgents($user_agents, $result);
		}
		else
		{
			$stats['os'] = BenutzerStatistik_Helper::parseOperatingSystems($user_agents);
			$stats['user_agents'] = BenutzerStatistik_Helper::parseUserAgents($user_agents);
		}

		return $stats;
	}

	private function getMonthNames()
	{
		return array(
			1=>_('Januar'),
			2=>_('Februar'),
			3=>_('Mrz'),
			4=>_('April'),
			5=>_('Mai'),
			6=>_('Juni'),
			7=>_('Juli'),
			8=>_('August'),
			9=>_('September'),
			10=>_('Oktober'),
			11=>_('November'),
			12=>_('Dezember')
		);
	}
}
?>